@@ -90,9 +90,9 @@ class Simulation(object):
         # when we have more information for this particular simulation
         self.__logger = logging.getLogger('Simulation')
 
-    # pylint: disable=too-many-locals
+    # pylint: disable=too-many-locals, too-many-arguments
     def launch(self, experiment_id, experiment_conf, server, reservation, cloned=True,
-               storage_token=None, brain_processes=1, profiler='disabled'):
+               storage_token=None, brain_processes=1, profiler='disabled', recordingPath=None):
         """
         Attempt to launch and initialize the given experiment on the given servers. This
         should not be directly invoked by users, use the VirtualCoach interface to validate
@@ -107,6 +107,7 @@ class Simulation(object):
         :param storage_token: A string representing the token for storage authorization.
         :param brain_processes: Number of mpi processes in the brain simulation.
         :param profiler: profiler option. Possible values are: disabled, cle_step and cprofile.
+        :param recordingPath: (optional) the path to a the zip of a recording.
         """
         assert isinstance(experiment_id, string_types)
         assert isinstance(experiment_conf, string_types)
@@ -142,7 +143,8 @@ class Simulation(object):
                         'experimentConfiguration': experiment_conf,
                         'gzserverHost': self.__server_info['serverJobLocation'],
                         'reservation': reservation,
-                        'private': cloned}
+                        'private': cloned,
+                        'playbackPath': recordingPath}
 
             status_code, sim_json = self.__http_client.post(url, sim_info)
 
@@ -277,7 +279,8 @@ class Simulation(object):
     def __on_error(self, msg):
         """
         Function to log ROS CLE error messages.
-        :param msg A CLEError message
+
+        :param msg: A CLEError message
         """
         error_msg = "There was a %(type)s error resulting from the %(source)s." \
                     " The full error is below:\n %(msg)s" % {'type': msg['errorType'],
