@@ -43,6 +43,8 @@ import tempfile
 import requests
 from urllib import parse as urlparse
 
+import secrets
+
 from copy import copy
 
 from datetime import datetime, timedelta
@@ -72,7 +74,7 @@ class VirtualCoach(object):
     """
 
     def __init__(self, environment='http://localhost:9000', oidc_username=None, oidc_password=None,
-                 oidc_token=None, storage_username=None, storage_password=None):
+                 storage_username=None, storage_password=None):
         """
         Instantiates the Virtual Coach by loading the configuration file and logging into OIDC for
         the given user. This will only fail if the config file is invalid or if the user
@@ -86,9 +88,6 @@ class VirtualCoach(object):
                               authentication and no token is provided.
         :param oidc_password: (optional) A string representing the OIDC Server password. If
                                  not supplied, the user will be prompted to enter a password.
-        :param oidc_token: (optional) A string representing the OIDC token for the current
-                              user, required if the selected environment requires OIDC
-                              and the username is not provided.
         :param storage_username: (optional) A string representing the Storage Server username. It is
                                  required if the user wants to have access to the storage server to
                                  clone experiments and launch cloned experiments.
@@ -97,7 +96,6 @@ class VirtualCoach(object):
         """
         assert isinstance(environment, (string_types, type(None)))
         assert isinstance(oidc_username, (string_types, type(None)))
-        assert isinstance(oidc_token, (string_types, type(None)))
         assert isinstance(storage_username, (string_types, type(None)))
 
         # parse and load the config file before any OIDC actions
@@ -107,19 +105,12 @@ class VirtualCoach(object):
 
         # authorize client into oidc or storage server
         token = ''
-        if oidc_username or oidc_token:
-            if oidc_username:
-                logger.info('Logging into OIDC as: %s', oidc_username)
-                if not oidc_password:
-                    oidc_password = getpass.getpass(prompt='input your OIDC password: ')
-                token = self.__get_oidc_token(oidc_username, oidc_password)
-            else:
-                if self.__oidc_token_is_valid(token):
-                    logger.info('Using provided token for OIDC server authorization')
-                    token = oidc_token
-                else:
-                    raise Exception('Provided OIDC token is invalid, VirtualCoach cannot be '
-                                    'instantiated.')
+        if oidc_username:
+            logger.info('Logging into OIDC as: %s', oidc_username)
+            if not oidc_password:
+                oidc_password = getpass.getpass(prompt='input your OIDC password: ')
+            token = self.__get_oidc_token(oidc_username, oidc_password)
+
         elif storage_username:
             logger.warning('No OIDC username supplied, simulation services will fail if OIDC is '
                            'enabled in this environment (%s).', environment)
@@ -141,32 +132,6 @@ class VirtualCoach(object):
         # if the config is valid and the login doesn't fail, we're ready
         logger.info('Ready.')
 
-    def __oidc_token_is_valid(self, token):
-        '''
-        check if a given oidc token is valid
-
-        :param token: token to validate as a string
-        '''
-
-        response = requests.post('https://services.humanbrainproject.eu/oidc/tokeninfo',
-                                 data=urlparse.urlencode({'access_token': token}),
-                                 headers={'Content-Type': 'application/x-www-form-urlencoded'})
-
-        if response.status_code != 200:
-            raise Exception('OIDC token validation failed, Status Code: %d'
-                            % response.status_code)
-
-        d = json.loads(response.content)
-        if 'valid' in d and not d['valid']:
-            logger.info('OIDC token not valid')
-            return False
-        elif 'expires_in' in d and int(d['expires_in']) < 1:
-            logger.info('OIDC token expired')
-            return False
-
-        logger.info('OIDC token valid')
-        return True
-
     def __get_oidc_token(self, user_name, password):
         """
         Attempts to acquire a oidc server token based on the provided credentials
@@ -176,18 +141,19 @@ class VirtualCoach(object):
         """
 
         # hbp oidc request
-        client_id = 'kamaji-python-client'
-        oauth_url = 'https://services.humanbrainproject.eu/oidc/'
+        client_id = 'nrp-frontend'
+        oauth_url = 'https://iam.ebrains.eu/auth/realms/hbp/protocol/openid-connect/'
 
         # construct request
         query = {
             'response_type': 'token',
             'client_id': client_id,
-            'redirect_uri': oauth_url + 'resources/oauth_code.html',
-            'prompt': 'consent',
+            'redirect_uri': 'https://neurorobotics.net',
+            'nonce': secrets.token_urlsafe(),
+            'scope': 'openid profile email group'
         }
 
-        parts = list(urlparse.urlparse(oauth_url + 'authorize'))
+        parts = list(urlparse.urlparse(oauth_url + 'auth'))
         query.update(dict(urlparse.parse_qsl(parts[4])))  # 4 is the index of the query part
         parts[4] = urlparse.urlencode(query)
         authorize_url = urlparse.urlunparse(parts)
@@ -198,13 +164,14 @@ class VirtualCoach(object):
         br.set_handle_robots(False)
         for _ in range(3):
             br.open(authorize_url)
-            br.select_form(name='j_spring_security_check')
+            br.select_form(nr=0)
 
             # fill form
             # pylint: disable=unsupported-assignment-operation
-            br['j_username'] = user_name
-            br['j_password'] = password
+            br['username'] = user_name
+            br['password'] = password
 
+            # pylint: disable=assignment-from-none
             res = br.submit()
 
             if 'error=' not in res.geturl():
@@ -214,8 +181,7 @@ class VirtualCoach(object):
 
         # the user is forwarded to the approve page if not approved yet
         if 'access_token' not in urlparse.urlparse(res.geturl()).fragment.lower():
-            br.select_form(name='confirmationForm')
-            res = br.submit()
+            raise Exception('Login to NRP online and give your consent. Then return to VC.')
 
         url_with_fragment = res.geturl()
 
@@ -249,8 +215,10 @@ class VirtualCoach(object):
 
         return token
 
-    def launch_experiment(self, experiment_id, server=None, reservation=None, cloned=True, brain_processes=1,
-                          profiler='disabled'):
+    # pylint: disable-msg=too-many-locals
+    def launch_experiment(self, experiment_id,
+                          server=None, reservation=None, cloned=True, brain_processes=1,
+                          profiler='disabled', recordingPath=None):
         """
         Attempts to launch a simulation with the given parameters. If no server is explicitly given
         then all available backend servers will be tried. Only cloned experiments to the Storage
@@ -265,7 +233,9 @@ class VirtualCoach(object):
         :param cloned: (optional) True or False depending on if the launched
                        is a cloned experiment or not.
         :param brain_processes: (optional) Number of mpi processes in the brain simulation.
-        :param profiler: (optional) profiler option. Possible values are: disabled, cle_step and cprofile.
+        :param profiler: (optional) profiler option.
+                         Possible values are: disabled, cle_step and cprofile.
+        :param recordingPath: (optional) the path to a the zip of a recording.
         """
         assert isinstance(experiment_id, string_types)
         assert isinstance(server, (string_types, type(None)))
@@ -306,8 +276,8 @@ class VirtualCoach(object):
         sim = Simulation(self.__http_client, self.__config, self)
         for server_i in servers:
             try:
-                if sim.launch(experiment_id, str(experiment_conf), str(server_i),
-                              reservation, cloned, self.__storage_token, brain_processes, profiler):
+                if sim.launch(experiment_id, str(experiment_conf), str(server_i), reservation,
+                        cloned, self.__storage_token, brain_processes, profiler, recordingPath):
                     return sim
 
             # pylint: disable=broad-except
@@ -525,149 +495,155 @@ class VirtualCoach(object):
         logger.info('Experiment "%s" cloned successfully', experiment_id)
         return res.content
 
-    def print_csv_experiment_runs(self, exp_id):
+    def print_experiment_runs_files(self, exp_id, file_type):
         """
-        Prints the list of ids of experiment runs that generated CSV files. The ids can be used
-        to retrieve the CSV data
+        Prints the list of ids of experiment runs that generated files. The ids can be used
+        to retrieve the data
 
-        :param exp_id: The experiment id for which to retrieve the list of CSV simulation runs
+        :param exp_id: The experiment id for which to retrieve the list of simulation runs
+        :param file_type: type of files ['csv', 'profiler']
         """
-        csv_files = self.__get_available_CSV_files(exp_id)
+        files = self.__get_available_files(exp_id, file_type)
 
         table = Texttable()
         table.header(['Run id', 'Date', 'Bytes'])
         table.set_cols_align(['r', 'c', 'r'])
 
-        for i, run_date in enumerate(sorted(csv_files.keys())):
-            run_size = sum(file['size'] for file in list(csv_files[run_date].values()))
+        for i, run_date in enumerate(sorted(files.keys())):
+            run_size = sum(file['size'] for file in list(files[run_date].values()))
             table.add_row([i, run_date, run_size])
 
         logger.info('List of simulation runs')
         print(table.draw())
 
-    def print_csv_experiment_run_files(self, exp_id, run_id):
+    def print_experiment_run_files(self, exp_id, file_type, run_id):
         """
-        Prints the list of CSV files for a given experiment run
+        Prints the list of files for a given experiment run
 
-        :param exp_id: The experiment id for which to retrieve the list of CSV files
-        :param run_id: The run id for which to retrieve the list of CSV files
+        :param exp_id: The experiment id for which to retrieve the list of files
+        :param file_type: type of files ['csv', 'profiler']
+        :param run_id: The run id for which to retrieve the list of files
         """
-        csv_files = self.__get_available_CSV_files(exp_id)
+        files = self.__get_available_files(exp_id, file_type)
 
         table = Texttable()
         table.header(['File', 'Size'])
         table.set_cols_align(['l', 'r'])
 
-        sorted_runs = sorted(csv_files.keys())
+        sorted_runs = sorted(files.keys())
         if not 0 <= run_id < len(sorted_runs):
             raise Exception('Could not find run %i, %i runs were found' %
                             (run_id, len(sorted_runs)))
 
-        for csv_file in list(csv_files[sorted_runs[run_id]].values()):
-            table.add_row([csv_file['name'], csv_file['size']])
+        for file in list(files[sorted_runs[run_id]].values()):
+            table.add_row([file['name'], file['size']])
 
         logger.info('Run %i list of files.', run_id)
         print(table.draw())
 
-    def print_csv_last_run_files(self, exp_id):
+    def print_last_run_files(self, exp_id, file_type):
         """
-        Prints the list of CSV files for the last run of an experiment
+        Prints the list of files for the last run of an experiment
 
-        :param exp_id: The experiment id for which to retrieve the list of CSV files
+        :param exp_id: The experiment id for which to retrieve the list of files
+        :param file_type: type of files ['csv', 'profiler']
         """
-        csv_files = self.__get_available_CSV_files(exp_id)
+        files = self.__get_available_files(exp_id, file_type)
 
         table = Texttable()
         table.header(['File', 'Size'])
         table.set_cols_align(['l', 'r'])
 
-        sorted_runs = sorted(csv_files.keys())
+        sorted_runs = sorted(files.keys())
         if not sorted_runs:
             raise Exception('Could not find any run')
 
-        for csv_file in list(csv_files[sorted_runs[-1]].values()):
-            table.add_row([csv_file['name'], csv_file['size']])
+        for file in list(files[sorted_runs[-1]].values()):
+            table.add_row([file['name'], file['size']])
 
         logger.info('Last run list of files')
         print(table.draw())
 
-    def get_csv_experiment_run_file(self, exp_id, run_id, file_name):
+    def get_experiment_run_file(self, exp_id, file_type, run_id, file_name):
         """
-        Retrieves a CSV file content
+        Retrieves a file content
 
         :param exp_id: The experiment id
+        :param file_type: type of files ['csv', 'profiler']
         :param run_id: The run id
         :param file_uuid: The file uuid
         """
-        csv_files = self.__get_available_CSV_files(exp_id)
-        sorted_runs = sorted(csv_files.keys())
+        files = self.__get_available_files(exp_id, file_type)
+        sorted_runs = sorted(files.keys())
         if not 0 <= run_id < len(sorted_runs):
             raise Exception('Could not find run %i, %i runs were found' %
                             (run_id, len(sorted_runs)))
 
-        if file_name not in csv_files[sorted_runs[run_id]]:
-            file_names = ', '.join(f['name'] for f in list(csv_files[sorted_runs[run_id]].values()))
+        if file_name not in files[sorted_runs[run_id]]:
+            file_names = ', '.join(f['name'] for f in list(files[sorted_runs[run_id]].values()))
             raise Exception('Could not find file \'%s\' in run %i, available file names are: %s' %
                             (file_name, run_id, file_names))
 
-        file_uuid = csv_files[sorted_runs[run_id]][file_name]['uuid']
-        return self.__get_csv_file_content(exp_id, file_uuid)
+        file_uuid = files[sorted_runs[run_id]][file_name]['uuid']
+        return self.__get_file_content(exp_id, file_uuid)
 
-    def get_csv_last_run_file(self, exp_id, file_name):
+    def get_last_run_file(self, exp_id, file_type, file_name):
         """
-        Retrieves a CSV file content for the last run
+        Retrieves a file content for the last run
 
         :param exp_id: The experiment id
+        :param file_type: type of files ['csv', 'profiler']
         :param file_name: The file name
         """
 
-        csv_files = self.__get_available_CSV_files(exp_id)
-        sorted_runs = sorted(csv_files.keys())
+        files = self.__get_available_files(exp_id, file_type)
+        sorted_runs = sorted(files.keys())
         if not sorted_runs:
             raise Exception('Could not find any run')
 
-        if file_name not in csv_files[sorted_runs[-1]]:
+        if file_name not in files[sorted_runs[-1]]:
             file_names = ', '.join(
-                file['name'] for file in list(csv_files[sorted_runs[-1]].values()))
+                file['name'] for file in list(files[sorted_runs[-1]].values()))
             raise Exception('Could not find file \'%s\' in last run, available file names are: %s' %
                             (file_name, file_names))
 
-        file_uuid = csv_files[sorted_runs[-1]][file_name]['uuid']
-        return self.__get_csv_file_content(exp_id, file_uuid)
+        file_uuid = files[sorted_runs[-1]][file_name]['uuid']
+        return self.__get_file_content(exp_id, file_uuid)
 
-    def __get_available_CSV_files(self, experiment_id):
+    def __get_available_files(self, experiment_id, file_type):
         """
-        Internal helper to retrieve the list of CSV files available for an experiment
+        Internal helper to retrieve the list of files available for an experiment
 
-        :param experiment_id: The experiment id for which to retrieve the list of CSV files
+        :param experiment_id: The experiment id for which to retrieve the list of files
+        :param file_type: type of files ['csv', 'profiler']
         """
-        response = requests.get(self.__config['proxy-services']['csv-files'] % (experiment_id,),
-                                headers=self.__http_headers)
+        response = requests.get(self.__config['proxy-services']['files'] % \
+                                (experiment_id, file_type,), headers=self.__http_headers)
 
         if response.status_code != http.client.OK:
-            raise Exception('Error when getting CSV files Status Code: %d. Error: %s'
+            raise Exception('Error when getting files Status Code: %d. Error: %s'
                             % (response.status_code, response))
-        csv_files = json.loads(response.content)
+        files = json.loads(response.content)
         distinct_runs = defaultdict(dict)
-        for csv_file in csv_files:
-            distinct_runs[csv_file['folder']][csv_file['name']] = csv_file
+        for file in files:
+            distinct_runs[file['folder']][file['name']] = file
         return distinct_runs
 
-    def __get_csv_file_content(self, exp_id, file_uuid):
+    def __get_file_content(self, exp_id, file_uuid):
         """
-        Internal helper method to retrieve a CSV file content
+        Internal helper method to retrieve a file content
 
-        :param exp_id: The experiment id for which to retrieve the CSV file content
+        :param exp_id: The experiment id for which to retrieve the file content
         :param file_uuid: The file uuid for which to retrieve the content
         """
-        logger.info('Retrieving CSV file.')
+        logger.info('Retrieving file.')
 
         response = requests.get(self.__config['proxy-services']['experiment-file'] %
                                 (exp_id, file_uuid),
                                 headers=self.__http_headers)
 
         if response.status_code != http.client.OK:
-            raise Exception('Error when getting CSV file Status Code: %d. Error: %s'
+            raise Exception('Error when getting file Status Code: %d. Error: %s'
                             % (response.status_code, response))
 
         return response.content
@@ -728,6 +704,110 @@ class VirtualCoach(object):
                             % (response.status_code, response))
         return response
 
+    def print_available_models(self, model_type, user="all"):
+        """
+        Prints available storage models.
+        :param model_type: type of the models to be shown ['environments', 'robots', 'brains']
+        :param user: username of models owner to be shown
+        """
+        assert isinstance(model_type, string_types)
+        assert isinstance(user, string_types)
+
+        if model_type not in ['robots', 'brains', 'environments']:
+            raise ValueError("Type must be a string in \
+                             ['robots', 'brains', 'environments']")
+
+        model_list = self.__get_model_list(model_type=model_type, user=user)
+
+        table = Texttable()
+        table.header(['Name', 'Display Name',
+                      'isShared', 'isCustom', 'Description'])
+        for name, v in sorted(iter(model_list.items()), key=lambda x: x[1]['name']):
+            table.add_row([name, v['displayName'], v['isShared'],
+                           v['isCustom'], v['description']])
+
+        # display the table
+        print(table.draw())
+
+    def import_model(self, path, model_type):
+        """
+        Imports a model (brain, robot or experiment as folder or zipped
+        folder) into user storage.
+        :param path: Path to the model folder or .zip file to be imported.
+        :param model_type: Model type to be imported ['environments', 'robots', 'brains']
+        """
+
+        assert isinstance(path, string_types)
+        assert isinstance(model_type, string_types)
+
+        if model_type not in ['robots', 'brains', 'environments']:
+            raise ValueError("Type must be a string in ['robots', \
+                             'brains', 'environments']")
+
+        if not os.path.isfile(path) and not os.path.isdir(path):
+            raise ValueError('The file or folder named %(path)s\
+                             does not exist.' % {'path': path})
+
+        if os.path.isdir(path):
+            # Handles a model folder
+            content = self.__get_directory_content(path)
+        else:
+            # Handles a zip file
+            try:
+                with open(path, 'rb') as f:
+                    content = f.read()
+            except Exception as e:
+                logger.error('The file %s could not be open', path)
+                raise e
+
+        file_headers = copy(self.__http_headers)
+        file_headers['Content-Type'] = 'application/octet-stream'
+        response = requests.post(self.__config['proxy-services']['storage-models-import'] %
+                                 (model_type, os.path.basename(path)),
+                                 data=content, headers=file_headers)
+
+        if response.status_code != requests.codes.ok:
+            raise Exception('Error when importing model: %d. Error: %s'
+                            % (response.status_code, response))
+
+        return response
+
+    def delete_model(self, model_type, name):
+        """
+        Deletes a model from user storage.
+        :param name: Name of the model to be deleted.
+        :param model_type: Model type to be deleted ['environments', 'robots', 'brains']
+        """
+        assert isinstance(model_type, string_types)
+        assert isinstance(name, string_types)
+
+        if model_type not in ['robots', 'brains', 'environments']:
+            raise ValueError("Type must be a string in ['robots',\
+                             'brains', 'environments']")
+
+        model_list = self.__get_model_list(model_type)
+        if name not in model_list:
+            raise ValueError('Model Name: "%s" is invalid, the model does not exist in your'
+                             ' storage. Please check the list of all models: \n%s'
+                             % (name, model_list.keys()))
+        self.__http_client.delete(self.__config['proxy-services']['storage-models-delete']
+                                  % (model_type, name,), body={})
+        logger.info('Model "%s" deleted successfully', name)
+
+    def __get_model_list(self, model_type, user="all"):
+        """
+        Internal helper to retrieve and parse the model list from the backend proxy.
+
+        :param model_type: type of the model ['environments', 'robots', 'brains']
+        :param user: username of model owner to be printed
+        """
+        response = requests.get(self.__config['proxy-services']['storage-models']
+                                % (user, model_type,), headers=self.__http_headers)
+
+        model_list = {model['name']: model for model in json.loads(response.content)}
+
+        return model_list
+
     @staticmethod
     def __zip_directory(dirpath, zip_filehandle):
         """
@@ -751,7 +831,7 @@ class VirtualCoach(object):
         Internal helper function
         It zips the target folder and returns its content
 
-        :param dirpath: path to the experiment folder to be zipped
+        :param dirpath: path to the experiment/model folder to be zipped
         """
         temp = tempfile.mktemp()
         zip_file = zipfile.ZipFile(temp, 'w', zipfile.ZIP_DEFLATED)
@@ -761,6 +841,7 @@ class VirtualCoach(object):
             logger.error('The folder %s could not be zipped', dirpath)
             raise e
         zip_file.close()
-        content = open(temp, 'rb').read()
+        with open(temp, 'rb') as f:
+            content = f.read()
         os.remove(temp)
         return content
